#!/tomcat/python3env/bin/python3
import os,sys,re
import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import seaborn as sns
from matplotlib.pyplot import figure, show
from scipy.stats import spearmanr, variation
import io
import cgi
import base64
from collections import OrderedDict
import gseapy
import statsmodels.stats.multitest as smt

form = cgi.FieldStorage() 
# input file, contains header, tab delimited
wo = os.getcwd()
# read the data

if not os.path.exists(wo+os.sep+"genecorrelation"):
    os.makedirs(wo+os.sep+"genecorrelation") 

omics1=pd.read_csv(wo+os.sep+sys.argv[1],sep="\t",header=0,index_col=0)
omics2=pd.read_csv(wo+os.sep+sys.argv[2],sep="\t",header=0,index_col=0)
omics1=omics1.apply (pd.to_numeric, errors='coerce').dropna()
omics2=omics2.apply (pd.to_numeric, errors='coerce').dropna()
omics1 = omics1.loc[~omics1.index.duplicated(keep='first'),~omics1.columns.duplicated(keep='first')]
omics2 = omics2.loc[~omics2.index.duplicated(keep='first'),~omics2.columns.duplicated(keep='first')]
# annotation=pd.read_csv(wo+os.sep+"omics3.txt",sep="\t",header=0,index_col=0)
geneinorder=set(omics1.index).intersection(set(omics2.index))
sampleinorder=set(omics1.columns).intersection(set(omics2.columns))

if len(geneinorder)>9 and len(sampleinorder)>9:
    omics2a=omics2.loc[list(geneinorder),list(sampleinorder)]
    omics1a=omics1.loc[list(geneinorder),list(sampleinorder)]
    omics1a.to_csv(wo+os.sep+"genecorrelation/omics1a.txt",sep="\t")
    omics2a.to_csv(wo+os.sep+"genecorrelation/omics2a.txt",sep="\t")    
    # omics2az=pd.DataFrame(np.power(2,omics2a),columns=omics2a.columns, index=omics2a.index)
    omics2az=pd.DataFrame(omics2a,columns=omics2a.columns, index=omics2a.index)
    omics1az=pd.DataFrame(omics1a,columns=omics1a.columns, index=omics1a.index)




    omics1aflat=pd.DataFrame(omics1a.values.flatten())
    omics2aflat=pd.DataFrame(omics2a.values.flatten())
    corr=[]
    for index, row in omics1a.iterrows():
        corr1,p=spearmanr(omics1a.loc[index,:].T,omics2a.loc[index,:].T)
        if omics1aflat.quantile(0.1)[0]<0 and omics1aflat.quantile(1)[0]<200:
            cv1 = variation(np.power(2,omics1a.loc[index,:]), axis=0)
        else:
            cv1 = variation(omics1a.loc[index,:], axis=0)
        if omics2aflat.quantile(0.1)[0]<0 and omics2aflat.quantile(1)[0]<200:
            cv2 = variation(np.power(2,omics2a.loc[index,:]), axis=0)
        else:
            cv2 = variation(omics2a.loc[index,:], axis=0)
        corr.append([index,corr1,p,cv1,cv2])
    #     print([index,corr1,p])
    df_corrgenewise=pd.DataFrame(np.asarray(corr))
    df_corrgenewise.apply(pd.to_numeric, errors='ignore', downcast='float')
    df_corrgenewise.columns=["Gene","Gene Correlation","P","CV1","CV2"]
    df_corrgenewise["Gene Correlation"]=df_corrgenewise["Gene Correlation"].astype(float)
    df_corrgenewise.set_index('Gene', inplace=True)
    df_corrgenewise['BH adjusted P'] =  smt.multipletests(df_corrgenewise['P'].astype(float), method='fdr_bh')[1]
    df_corrgenewise.sort_values(by="Gene Correlation",inplace=True,ascending=False)
    df_corrgenewise.to_csv(wo+os.sep+"genecorrelation/gene_wise_corr.txt",sep="\t")
    # In[125]:

    fig = plt.figure(figsize=(4,4))
    sns.set_style("white")
    ax=sns.distplot(df_corrgenewise["Gene Correlation"],color="orangered");
    plt.title('Gene-wise correlation')
    ax.set_xlabel("Spearman's correlation",size=11,rotation=0)
    ax.set_ylabel("Probability density",size=11,rotation=90)
    mediancorr=df_corrgenewise["Gene Correlation"].median()
    plt.text(-0.45, 1.4, "Median={0:.2f}".format(mediancorr),fontsize=12)
    plt.xticks(size=11)
    plt.yticks(size=11)
    plt.xlim((-0.5,1))
    plt.ylim((0,2.3))
    plt.savefig(wo+os.sep+"genecorrelation/genecorrelation.png",dpi=100,bbox_inches = 'tight')


    sstemp2 = gseapy.prerank(rnk=df_corrgenewise.loc[:,"Gene Correlation"], 
                         gene_sets="/var/lib/tomcat8/webapps/OmicsX/c2.cp.kegg.v6.2.symbols.gmt",
    #                     gene_sets='KEGG_2016',
                         processes=4,
                         permutation_num=100, # reduce number to speed up test
                         outdir=wo+os.sep+'genecorrelation/gsea1',format='png',
                             )



