#!/tomcat/python3env/bin/python3
import os,sys,re
import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import seaborn as sns
from matplotlib.pyplot import figure, show
from scipy.stats import spearmanr, variation,ttest_ind
import io
import cgi
import base64
from collections import OrderedDict
import gseapy
import statsmodels.stats.multitest as smt

form = cgi.FieldStorage() 
# input file, contains header, tab delimited
wo = os.getcwd()
# read the data
if not os.path.exists(wo+os.sep+"groupdifference"):
    os.makedirs(wo+os.sep+"groupdifference") 

group1name = sys.argv[1]
group2name = sys.argv[2]


omics1=pd.read_csv(wo+os.sep+sys.argv[3],sep="\t",header=0,index_col=0)
omics2=pd.read_csv(wo+os.sep+sys.argv[4],sep="\t",header=0,index_col=0)
omics1=omics1.apply (pd.to_numeric, errors='coerce').dropna()
omics2=omics2.apply (pd.to_numeric, errors='coerce').dropna()
omics1 = omics1.loc[~omics1.index.duplicated(keep='first'),~omics1.columns.duplicated(keep='first')]
omics2 = omics2.loc[~omics2.index.duplicated(keep='first'),~omics2.columns.duplicated(keep='first')]
# annotation=pd.read_csv(wo+os.sep+"omics3.txt",sep="\t",header=0,index_col=0)
geneinorder=set(omics1.index).intersection(set(omics2.index))
sampleinorder=set(omics1.columns).intersection(set(omics2.columns))

if len(geneinorder)>9 and len(sampleinorder)>9:
    omics2a=omics2.loc[list(geneinorder),list(sampleinorder)]
    omics1a=omics1.loc[list(geneinorder),list(sampleinorder)]
    omics1a.to_csv(wo+os.sep+"groupdifference/omics1a.txt",sep="\t")
    omics2a.to_csv(wo+os.sep+"groupdifference/omics2a.txt",sep="\t")
    # omics2az=pd.DataFrame(np.power(2,omics2a),columns=omics2a.columns, index=omics2a.index)
    omics2az=pd.DataFrame(omics2a,columns=omics2a.columns, index=omics2a.index)
    omics1az=pd.DataFrame(omics1a,columns=omics1a.columns, index=omics1a.index)

annotation=pd.read_csv(wo+os.sep+sys.argv[5],sep="\t",header=0,index_col=0)
m=-1
annotationcol=[]
for index,row in annotation.iterrows():
    m=m+1
    for label, content in annotation.items():
        annotationcol.append([label,annotation.loc[index,label]])
annotationgroup=pd.DataFrame(np.asarray(annotationcol))
annotationgroup.columns=["sample","group"]



group1a=list(annotationgroup[annotationgroup["group"]==group1name].loc[:,"sample"])
group2a=list(annotationgroup[annotationgroup["group"]==group2name].loc[:,"sample"])

geneinorder=set(omics1.index).intersection(set(omics2.index))
sampleinorder=set(omics1.columns).intersection(set(omics2.columns))
group1=set(group1a).intersection(list(sampleinorder))
group2=set(group2a).intersection(list(sampleinorder))
omics2a=omics2.loc[list(geneinorder),list(sampleinorder)]
omics1a=omics1.loc[list(geneinorder),list(sampleinorder)]
omics2az=pd.DataFrame(omics2a,columns=omics2a.columns, index=omics2a.index)
omics1az=pd.DataFrame(omics1a,columns=omics1a.columns, index=omics1a.index)


from scipy import stats
import math
from scipy.stats import variation 
def ttest_fc(g6a,tumor,normal):
    fc_list=[]
    t_list=[]
    p_list=[]
    g6a=g6a.astype("float")
  
    for index, row in g6a.iterrows():
        tr=row[tumor]
        nl=row[normal]
#         fc=np.median([i-j for i , j in zip (tr , nl)])
#         t,p=stats.ttest_ind(tr.values, nl.values, equal_var = False)
        fc=np.mean(tr)- np.mean(nl)
        t,p=stats.ttest_ind(tr.values, nl.values, equal_var = False)
        fc_list.append(fc)
        t_list.append(t)
        p_list.append(p)
    g6_diff=g6a.copy(deep=True)
    g6_diff["prlog2fc"]=fc_list
    g6_diff["prt"]=t_list
    g6_diff["prp"]=p_list
    return g6_diff

def ttest_fcraw(g6a,tumor,normal):
    fc_list=[]
    t_list=[]
    p_list=[]
    g6a=g6a.astype("float")
  
    for index, row in g6a.iterrows():
        tr=row[tumor]
        nl=row[normal]
#         fc=np.median([i-j for i , j in zip (tr , nl)])
#         t,p=stats.ttest_ind(tr.values, nl.values, equal_var = False)
        fc=math.log((np.mean(tr)+0.0001)/(np.mean(nl)+0.0001),2)
        t,p=ttest_ind(tr.values, nl.values, equal_var = False)
        fc_list.append(fc)
        t_list.append(t)
        p_list.append(p)
    g6_diff=g6a.copy(deep=True)
    g6_diff["prlog2fc"]=fc_list
    g6_diff["prt"]=t_list
    g6_diff["prp"]=p_list
    return g6_diff

omics1azflat=pd.DataFrame(omics1az.values.flatten())
omics2azflat=pd.DataFrame(omics2az.values.flatten())
if omics1azflat.quantile(0.1)[0]<0 and omics1azflat.quantile(1)[0]<200:
    omics1az_diff=ttest_fc(omics1az,group1,group2)
else:
    omics1az_diff=ttest_fcraw(omics1az,group1,group2)
if omics2azflat.quantile(0.1)[0]<0 and omics2azflat.quantile(1)[0]<200:
    omics2az_diff=ttest_fc(omics2az,group1,group2)
else:
    omics2az_diff=ttest_fcraw(omics2az,group1,group2)


# In[115]:

omicsdifference=pd.concat([pd.DataFrame(omics1az_diff.loc[:,"prlog2fc"]),pd.DataFrame(omics2az_diff.loc[:,"prlog2fc"])], axis=1)
omicsdifference.columns = ["log2 fold change in omics1", "log2 fold change in omics2"]

omicsdifference["groupdifference"]=abs(omicsdifference["log2 fold change in omics1"]-omicsdifference["log2 fold change in omics2"])
omicsdifference.sort_values(by="groupdifference",inplace=True,ascending=False)
omicsdifference.to_csv(wo+os.sep+"groupdifference/groupdifference.txt",sep="\t")

# In[117]:

sstemp2 = gseapy.prerank(rnk=omicsdifference["groupdifference"], 
                     gene_sets="/var/lib/tomcat8/webapps/OmicsX/c2.cp.kegg.v6.2.symbols.gmt",
#                     gene_sets='KEGG_2016',
                     processes=10,
                     permutation_num=100, # reduce number to speed up test
                     outdir=wo+os.sep+"groupdifference/"+group1name+"_"+group2name+'_gsea3',format='png',
                         )


fig, ax = plt.subplots(figsize=(4, 4))
ax=plt.scatter(x="log2 fold change in omics1", y="log2 fold change in omics2", data=omicsdifference,c='silver', s=20)


# plt.xlim(-40, 40)
# plt.xticks(())  # ignore xticks
# plt.ylim(-40, 40)

# plt.legend(["All","ribosome","Glycolysis","TCA cycle","ccc"],fontsize=12)
plt.axvline(x=0, color='k')
plt.axhline(y=0, color='k')
plt.xlabel("log2 fold change in omics1",fontsize=10)
plt.ylabel("log2 fold change in omics2",fontsize=10)
# plt.text(globalrna.loc["COX6C","global_prt"], globalrna.loc["COX6C",'rna_prt'], "COX6C",fontsize=12)
# plt.text(globalrna.loc["ALDOB","global_prt"], globalrna.loc["ALDOB",'rna_prt'], "PFKL",fontsize=12)
# plt.text(globalrna.loc["NDUFS1","global_prt"], globalrna.loc["NDUFS1",'rna_prt'], "NDUFS1",fontsize=12)



plt.yticks(size=12)  # ignore yticks
plt.xticks(size=12)
plt.savefig(wo+os.sep+"groupdifference/"+group1name+"_"+group2name+"_difference.png",dpi=100,bbox_inches = 'tight')

