#!/tomcat/python3env/bin/python3
import os,sys,re
import numpy as np
import pandas as pd
import matplotlib
matplotlib.use('Agg')
import matplotlib.pyplot as plt
import math
import seaborn as sns
from matplotlib.pyplot import figure, show
from scipy.stats import spearmanr, variation
import io
import cgi
import base64
from collections import OrderedDict
import gseapy
import statsmodels.stats.multitest as smt
from scipy import stats
import catheat
from catheat import plot as cplt

form = cgi.FieldStorage() 
# input file, contains header, tab delimited
wo = os.getcwd()
# read the data

if not os.path.exists(wo+os.sep+"samplecorrelation"):
    os.makedirs(wo+os.sep+"samplecorrelation") 
wi="/var/lib/tomcat8/webapps/OmicsX/"

omics1=pd.read_csv(wo+os.sep+sys.argv[1],sep="\t",header=0,index_col=0)
omics2=pd.read_csv(wo+os.sep+sys.argv[2],sep="\t",header=0,index_col=0)
omics1=omics1.apply (pd.to_numeric, errors='coerce').dropna()
omics2=omics2.apply (pd.to_numeric, errors='coerce').dropna()
omics1 = omics1.loc[~omics1.index.duplicated(keep='first'),~omics1.columns.duplicated(keep='first')]
omics2 = omics2.loc[~omics2.index.duplicated(keep='first'),~omics2.columns.duplicated(keep='first')]
# annotation=pd.read_csv(wo+os.sep+"omics3.txt",sep="\t",header=0,index_col=0)
geneinorder=set(omics1.index).intersection(set(omics2.index))
sampleinorder=set(omics1.columns).intersection(set(omics2.columns))

if len(geneinorder)>9 and len(sampleinorder)>9:
    omics2a=omics2.loc[list(geneinorder),list(sampleinorder)]
    omics1a=omics1.loc[list(geneinorder),list(sampleinorder)]
    omics1a.to_csv(wo+os.sep+"samplecorrelation/omics1a.txt",sep="\t")
    omics2a.to_csv(wo+os.sep+"samplecorrelation/omics2a.txt",sep="\t")    
    # omics2az=pd.DataFrame(np.power(2,omics2a),columns=omics2a.columns, index=omics2a.index)
    omics2az=pd.DataFrame(omics2a,columns=omics2a.columns, index=omics2a.index)
    omics1az=pd.DataFrame(omics1a,columns=omics1a.columns, index=omics1a.index)
#annotation=pd.read_csv(wi+os.sep+"tumorClinical.txt",sep="\t",header=0,index_col=0)

if len(sys.argv)>3:
    annotation=pd.read_csv(wo+os.sep+sys.argv[3],sep="\t",header=0,index_col=0)
 

corr=[]  
for each in omics1a.columns:
    corr1,p=spearmanr(omics1a[each],omics2a[each])
#     if each in omics1a.columns:
#         m=0
#     else:
#         m=1
    corr.append([each,corr1,p])
df_samplecorr=pd.DataFrame(np.asarray(corr))  
df_samplecorr.columns=["Sample","Corr","p-value"]
df_samplecorr.set_index('Sample', inplace=True)
df_samplecorr["Corr"]=df_samplecorr["Corr"].astype(float)
df_samplecorr.sort_values(by="Corr",inplace=True,ascending=False)
df_samplecorr.to_csv(wo+os.sep+"samplecorrelation/sample_wise_correlation.txt",sep="\t")

if os.path.exists(wo+os.sep+"omics3.txt") and (os.stat(wo+os.sep+"omics3.txt").st_size>1):
	newcolumn=[]
	for each in annotation.index:
	    colname=each
	    for value in annotation.T[each].unique():
	        group1a=list(annotation.T[annotation.T[each]==value].index)
	        group1b=list(annotation.T[annotation.T[each]!=value].index)
	        t,p=stats.ttest_ind(df_samplecorr.loc[group1a]["Corr"].values, df_samplecorr.loc[group1b]["Corr"].values, equal_var = False)
	        if(p<0.05):
	            colname=each+"(*)"
	            break
	    newcolumn.append(colname)
        


# In[70]:

fig = plt.figure(figsize=(7,4))
sns.set_style("white")
ax=fig.add_axes([0.15,0.3,0.7,0.5])

sns.barplot(y="Corr",x=df_samplecorr.index, data=df_samplecorr,
            color="b",alpha=1)
plt.legend(ncol=2, loc="upper left", frameon=False,fontsize=10)
# plt.ylim(0.342,0.56)
# plt.xlim(-0.5,71.5)
ax.axes.get_xaxis().set_visible(False)
# ax.set_fontsize(13)
plt.title('Sample-wise mRNA-protein correlation',fontsize=10)
plt.ylabel("Index (Spearman correlation)",fontsize=10)
plt.yticks(fontsize=10)


if os.path.exists(wo+os.sep+"omics3.txt") and (os.stat(wo+os.sep+"omics3.txt").st_size>1):
    if len(annotation)<3:
        ax4=fig.add_axes([0.15,0.2,0.7,0.05])
    elif len(annotation)<6:
        ax4=fig.add_axes([0.15,0.1,0.7,0.1])
    else:
        ax4=fig.add_axes([0.15,0.05,0.7,0.2])
    cm4={"G1":"palegreen",'G2':"mediumspringgreen","G3":"mediumseagreen","G4":"darkgreen","QC":"blue","NCI7":"purple","Normal":"#f0f5f5","nan":"gray",}
    pc=cplt.heatmap( annotation.loc[:,df_samplecorr.index],cmap=cm4,ax=ax4,
                legend=False,xticklabels = False,yticklabels = True,linewidth=0.2,linecolor="white",
               cbar_kws={'label': 'Accuracy %'})
    ax4.set_yticklabels(newcolumn, rotation=0, fontsize=10)
    plt.ylabel('')
plt.savefig(wo+os.sep+"samplecorrelation/figure1b.png",dpi=100,bbox_inches = 'tight')


corr=[]
for index, row in omics2a.iterrows():
    corr1,p1=spearmanr(df_samplecorr.loc[omics2a.columns,"Corr"],omics1a.loc[index,:].T)
    corr2,p2=spearmanr(df_samplecorr.loc[omics2a.columns,"Corr"],omics2a.loc[index,:].T)
    corr.append([index,corr1,p1,corr2,p2])
df_corrRNAproteinGene=pd.DataFrame(np.asarray(corr)) 
df_corrRNAproteinGene.columns=["Gene","Corr_omics1","P_omics1","Corr_omics2","P_omics2"]
df_corrRNAproteinGene.set_index('Gene', inplace=True)
df_corrRNAproteinGene["Corr_omics1"]=df_corrRNAproteinGene["Corr_omics1"].astype(float)
df_corrRNAproteinGene["Corr_omics2"]=df_corrRNAproteinGene["Corr_omics2"].astype(float)
df_corrRNAproteinGene.sort_values(by="Corr_omics1",inplace=True,ascending=False)
df_corrRNAproteinGene.to_csv(wo+os.sep+"samplecorrelation/sample_wise_corr.txt",sep="\t")



sstemp2 = gseapy.prerank(rnk=df_corrRNAproteinGene["Corr_omics1"], 
                     gene_sets=wi+os.sep+"/c2.cp.kegg.v6.2.symbols.gmt",
#                     gene_sets='KEGG_2016',
                     processes=8,
                     permutation_num=100, # reduce number to speed up test
                     outdir=wo+os.sep+'samplecorrelation/gsea2a',format='png',
                         )
df_corrRNAproteinGene.sort_values(by="Corr_omics2",inplace=True,ascending=False)

sstemp3 = gseapy.prerank(rnk=df_corrRNAproteinGene["Corr_omics2"], 
                     gene_sets=wo+os.sep+"c2.cp.kegg.v6.2.symbols.gmt",
#                     gene_sets='KEGG_2016',
                     processes=8,
                     permutation_num=100, # reduce number to speed up test
                     outdir=wo+os.sep+'samplecorrelation/gsea2b',format='png',
                         )


# In[71]:
